#include <ch.h>
#include <hal.h>
#include "buttons.h"

/* Constants */
static const ioline_t button_Lines[BUTTON_NUMS] = {
  LINE_BUTTON1, LINE_BUTTON2, LINE_BUTTON3, LINE_BUTTON4
};
static const eventmask_t button_Events[BUTTON_NUMS] = {
  EVT_BTN1_PRS, EVT_BTN2_PRS, EVT_BTN3_PRS, EVT_BTN4_PRS
};

/* Variables */
static thread_t * evt_thread;
static virtual_timer_t button_vt;
static button_state_t button_States[BUTTON_NUMS] = {
  BTN_st_Clear, BTN_st_Clear, BTN_st_Clear, BTN_st_Clear
};
static uint8_t button_Time[BUTTON_NUMS] = {0, 0, 0, 0};

/* Functions */
static void button_Process(virtual_timer_t *vtp, void *p) {
  (void)vtp;
  (void)p;
  eventmask_t events = 0;
  static int pause = 0;

  if (pause != 0) {
    pause --;
  } else {

    int i;
    for (i=0; i<BUTTON_NUMS; i++) {

      if (palReadLine(button_Lines[(button_num_t)i]) == BUTTON_PRESSED) {
      /* button pressed */
        button_Time[i] ++;
        if (button_Time[i] >= (BTN_TIME_HOLDED/BTN_SCAN_PERIOD)) {
          /* process long press */
          button_Time[i] -= (BTN_TIME_REPEATED/BTN_SCAN_PERIOD);
          button_States[i] = BTN_st_Holded;
          events |= button_Events[i]; // autorepeat
        }

      } else if (button_Time[i] != 0) {
      /* button released */
        if (button_Time[i] >= (BTN_TIME_PRESSED/BTN_SCAN_PERIOD)) {
          /* process short press */
          button_States[i] = BTN_st_Pressed;
          events |= button_Events[i];
        }
        button_Time[i] = 0;
        button_States[i] = BTN_st_Released;
        pause = (BTN_SCAN_PAUSE/BTN_SCAN_PERIOD);
      }

      /* Segnaling events to the thread, if any. */
      if (events) {
        chSysLockFromISR();
        chEvtSignalI(evt_thread, events);
        chSysUnlockFromISR();
        events = 0;
      }
    } /* end FOR */

  } /* end Pause check */
}

void buttons_Init(thread_t * thread) {
  evt_thread = thread;
  chVTSetContinuous(&button_vt, TIME_MS2I(BTN_SCAN_PERIOD), button_Process, NULL);
}

button_state_t buttons_GetState(const button_num_t btn) {
  return button_States[btn];
}